#include "arona_application.h"

#include <gdkmm/display.h>
#include <gdkmm/monitor.h>
#include <giomm/listmodel.h>

#include "spdlog/spdlog.h"
#include "thread_message/thread_message_hubs.h"
#include "tool/delta_timer.h"
#include "window/window_manager.h"

application* g_application = nullptr;

arona_application::arona_application() {
    g_application = this;
}

void arona_application::init() {
    application::init();
    // Get the screen refresh rate
    auto display = Gdk::Display::get_default();
    auto list_model = display->get_monitors();
    auto monitors = display->get_monitors();
    for (unsigned int i = 0; i < monitors->get_n_items(); ++i)
    {
        auto item = monitors->get_object(i);
        Glib::RefPtr<Gdk::Monitor> monitor = std::dynamic_pointer_cast<Gdk::Monitor>(item);
        screen_refresh_rate_ = std::max(screen_refresh_rate_, monitor->get_refresh_rate());
    }
    screen_refresh_rate_ /= 1000; // Convert to Hz
    // 在macos上可能无法获取屏幕刷新率
    if (screen_refresh_rate_ == 0) {
        screen_refresh_rate_ = 120;
        spdlog::error("Failed to get screen refresh rate, using default value {} Hz", screen_refresh_rate_);
    }
    if (screen_refresh_rate_ < 60) {
        spdlog::warn("Screen refresh rate is too low: {} Hz, will set to 60 hz", screen_refresh_rate_);
        screen_refresh_rate_ = 60;
    }
    spdlog::info("Screen refresh rate: {} Hz", screen_refresh_rate_);
    timer_ = Glib::signal_idle().connect(sigc::mem_fun(*this, &arona_application::update_application));
    // timer_ = Glib::signal_timeout().connect(&update_window_manager, 0);
}

void arona_application::shutdown() {
    timer_.disconnect();
    application::shutdown();
}

int32_t arona_application::get_screen_refresh_rate() const {
    return screen_refresh_rate_;
}


arona_application* get_arona_application() {
    return static_cast<arona_application*>(g_application);
}
