#pragma once
#include <gtkmm/drawingarea.h>
#include <gtkmm/eventcontroller.h>
#include <gtkmm/gesturedrag.h>
#include "spdlog/spdlog.h"

template<typename T>
class w_slider : public Gtk::DrawingArea {
public:
    explicit w_slider(bool vertical = false);
    void set_value(T value) {
        value_ = std::clamp(value, min_, max_);
        queue_draw();
    }
    void set_value_range(T min, T max) {
        min_ = min;
        max_ = max;
    }

    sigc::signal<void(T)> signal_value_changed() {
        return signal_value_changed_;
    }
    const bool is_vertical;
private:
    void on_draw(const Cairo::RefPtr<Cairo::Context>& cr, int width, int height);
    void on_begin_drag(double x, double y);
    void on_drag(double x, double y);
    T value_ = T{};
    T min_ = T{};
    T max_ = T{};
    sigc::signal<void(T)> signal_value_changed_;
    T begin_drag_value_ = T{};
};

template<typename T>
void w_slider<T>::on_begin_drag(double x, double y) {
    begin_drag_value_ = value_;
}

template<typename T>
void w_slider<T>::on_drag(double x, double y) {
    if (is_vertical) {
        double delta = y / get_allocated_height();
        set_value(begin_drag_value_ - delta);
    } else {
        double delta = x / get_allocated_width();
        set_value(begin_drag_value_ + delta);
    }
    signal_value_changed_.emit(value_);
}

template<typename T>
w_slider<T>::w_slider(bool vertical) : is_vertical(vertical) {
    set_content_height(10);
    set_content_width(10);
    set_draw_func(sigc::mem_fun(*this, &w_slider::on_draw));
    auto controller = Gtk::GestureDrag::create();
    controller->signal_drag_begin().connect(sigc::mem_fun(*this, &w_slider::on_begin_drag));
    controller->signal_drag_update().connect(sigc::mem_fun(*this, &w_slider::on_drag));

    add_controller(controller);
}

template<typename T>
void w_slider<T>::on_draw(const Cairo::RefPtr<Cairo::Context> &cr, int width, int height) {
    cr->set_source_rgb(0.8, 0.8, 0.8);
    cr->rectangle(0, 0, width, height);
    cr->fill();
    cr->set_source_rgb(0.5, 0.5, 0.5);
    if (is_vertical) {
        cr->rectangle(0, height * (1 - (value_ - min_) / (max_ - min_)), width, height * (value_ - min_) / (max_ - min_));
    } else {
        cr->rectangle(0, 0, width * (value_ - min_) / (max_ - min_), height);
    }
    cr->fill();
}
