#include "w_mixer_list.h"

#include "w_mixer_track.h"
#include "audio/mixer/mixer.h"
#include "thread_message/thread_message_hubs.h"

w_mixer_list::w_mixer_list() {
    mixer* mixer = get_mixer();

    {
        dummy_track *master_track = get_mixer()->get_master();
        master_track_ = Gtk::make_managed<w_mixer_track>();
        master_track_->set_mixer_track(master_track);
        append(*master_track_);
    }

    mixer->on_add_track.add_raw(this, &w_mixer_list::on_add_track);
    mixer->on_remove_track.add_raw(this, &w_mixer_list::on_remove_track);

    data_model_ = Gio::ListStore<mixer_list_model>::create();
    selection_model_ = Gtk::SingleSelection::create(data_model_);
    selection_model_->set_autoselect(true);

    factory_ = Gtk::SignalListItemFactory::create();
    factory_->signal_setup().connect(mem_fun(*this, &w_mixer_list::on_setup_track_widget));
    factory_->signal_bind().connect(mem_fun(*this, &w_mixer_list::on_bind_track_widget));

    list_view_.set_model(selection_model_);
    list_view_.set_factory(factory_);
    list_view_.set_orientation(Gtk::Orientation::HORIZONTAL);

    append(list_view_);
}

w_mixer_list::~w_mixer_list() {
    mixer* mixer = get_mixer();
    mixer->on_add_track.remove_object(this);
    mixer->on_remove_track.remove_object(this);
}

void w_mixer_list::on_add_track(mixer_track* mixer_track) {
    data_model_->append(mixer_list_model::create(mixer_track));
}

void w_mixer_list::on_remove_track(mixer_track* mixer_track) {
    for (int i = 0; i < data_model_->get_n_items(); ++i) {
        if (data_model_->get_item(i)->mixer == mixer_track) {
            data_model_->remove(i);
            return;
        }
    }
}

void w_mixer_list::on_setup_track_widget(const Glib::RefPtr<Gtk::ListItem>& item) {
    const auto widget = Gtk::make_managed<w_mixer_track>();
    item->set_child(*widget);
}

void w_mixer_list::on_bind_track_widget(const Glib::RefPtr<Gtk::ListItem>& item) {
    const auto index = item->get_position() + 1; // 0 is master track
    if (auto* widget = dynamic_cast<w_mixer_track*>(item->get_child())) {
        const std::vector<mixer_track*>& tracks = get_mixer()->get_tracks();
        if (index >= tracks.size()) {
            return;
        }
        widget->set_mixer_track(tracks[index]);
    }
}

void w_mixer_list::on_selection_changed() {
    const guint selected = selection_model_->get_selected();
    const Glib::RefPtr<mixer_list_model>& selected_model = data_model_->get_item(selected);
    if (!selection_model_) {
        return;
    }
    const mixer_track* track = selected_model->mixer;
    spdlog::info("Selected track: {}", track->get_name());
}
