#include "w_mixer_track.h"

#include "audio/mixer/mixer_track.h"
#include <gtkmm/adjustment.h>

std::string vertical_text(const std::string& text) {
    std::string result;
    for (char c : text) {
        result += c;
        result += '\n';
    }
    return result;
}

w_mixer_track::w_mixer_track() : Box(Gtk::Orientation::HORIZONTAL), track_(nullptr), volume_scale_(true) {
    auto volume_adjustment = Gtk::Adjustment::create(0.0, 0.0, 1.2, 0.01, 0.1, 0.0);
    volume_scale_.set_value_range(0.0, 1.2);
    volume_scale_.signal_value_changed().connect(sigc::mem_fun(*this, &w_mixer_track::on_volume_scale_value_changed));
    append(volume_scale_);
    append(audio_buffer_bar_);
    append(label_);
}

void w_mixer_track::set_mixer_track(mixer_track* track) {
    track_ = track;
    volume_scale_.set_value(track_->get_volume());
    audio_buffer_bar_.set_buffers(track_->ui_buffers);
    label_.set_text(vertical_text(track_->get_name().c_str()));
}

void w_mixer_track::on_volume_scale_value_changed(float value) {
    if (track_)
        track_->set_volume(value);
}
